//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "UnitJBForm.h"

namespace Unitjbform
{

void __fastcall TJBForm::SetMargin(const TPercent Value)
{
  FMargin = Value;
}

void __fastcall TJBForm::SetSnapToEdge(const bool Value)
{
  FMoveFormByControls = Value;
}

void __fastcall TJBForm::SetShowCaption(const bool Value)
{
  FShowCaption = Value;
  if (!FShowCaption)
  {
    SetWindowLong(Handle, GWL_STYLE, GetWindowLong(Handle, GWL_STYLE) & ~WS_CAPTION );
    Height -= GetSystemMetrics(SM_CYCAPTION);
  }
  else
  {
    SetWindowLong(Handle,GWL_STYLE, GetWindowLong(Handle, GWL_STYLE) | WS_CAPTION);
    Height += GetSystemMetrics(SM_CYCAPTION);
  }
}

void __fastcall TJBForm::SetTransparency(const int Value)
{
  if (FTransparency == Value)
    return;

  FTransparency = Value;
  SetLayeredWindowAttributes(Handle, 0, FTransparency, LWA_ALPHA);
}

void __fastcall TJBForm::SetUseTransparency(const bool Value)
{
  FUseTransparency = Value;
}

void __fastcall TJBForm::SetMoveFormByControls(const bool Value)
{
  FMoveFormByControls = Value;
}


__fastcall TJBForm::TJBForm(Classes::TComponent* AOwner) :TForm(AOwner)
{
  SetWindowLong(Handle, GWL_EXSTYLE, GetWindowLong(Handle,GWL_EXSTYLE) | WS_EX_LAYERED);
  Transparency = 255;
  FUseTransparency = true;
  FShowCaption = true;
  FMargin = 2;
  FSnapToEdge = true;
  FMoveFormByControls = true;

  TMyControlSubclass *FSubclass = new TMyControlSubclass();

  RegisterSubclass(this, FSubclass);

  for (int i = 0; i<ComponentCount; ++i)
    if (dynamic_cast<TControl*>(Components[i])!=NULL)
       RegisterSubclass(dynamic_cast<TControl*>(Components[i]), FSubclass);
}

__fastcall TJBForm::~TJBForm(void)
{
  for (int i = 0; i<ComponentCount; ++i)
    if (dynamic_cast<TControl*>(Components[i])!=NULL)
       UnRegisterSubclass(dynamic_cast<TControl*>(Components[i]), FSubclass);

  UnRegisterSubclass(this, FSubclass);
  delete FSubclass;
}


TJBForm* __fastcall TMyControlSubclass::GetParentForm(Controls::TControl* Control)
{
  return  (TJBForm *)Forms::GetParentForm(Control);
}

unsigned __fastcall TMyControlSubclass::SetTimer(HWND hWnd, unsigned nIDEvent, unsigned uElapse, TIMERPROC lpTimerFunc)
{

  switch (nIDEvent)
  {
    case 1:
      Timer1 = true;
      break;
    case 2:
      Timer2 = true;
      break;
  }
  return ::SetTimer(hWnd, nIDEvent, uElapse, lpTimerFunc);
}

BOOL __fastcall TMyControlSubclass::KillTimer(HWND hWnd, unsigned uIDEvent)
{
  switch (uIDEvent)
  {
    case 1:
      Timer1 = false;
      break;
    case 2:
      Timer2 = false;
      break;
  }
  return ::KillTimer(hWnd, uIDEvent);
}

void __fastcall TMyControlSubclass::BeforeWndProc(Controls::TControl* Control, Messages::TMessage &Message)
{
  JBForm = GetParentForm(Control);
  if (dynamic_cast<TComponent*>(Control)!=NULL)
    if (dynamic_cast<TComponent*>(Control)->Tag != 0)
      return;


  switch (Message.Msg)
  {
    case WM_TIMER :
    {
      switch (Message.WParam)
      {
        case 1:
          doBlendDown();
          break;
        case 2:
          doBlendUp();
          break;
      }
     break;
    }

    case WM_NCLBUTTONDOWN :
    {
      doMouseDown();
      break;
    }
    case WM_NCMOUSEMOVE :
    {
      doNcMouseMove();
      doMouseUp();
      break;
    }
  }

  if (! JBForm->FMoveFormByControls)
    return;

  switch (Message.Msg)
  {
    case WM_LBUTTONDOWN :
      doMouseDown();
      break;
    case WM_LBUTTONUP :
      doMouseUp();
      break;
    case WM_MOUSEMOVE:
      doMouseMove();
      break;
  }
}

void __fastcall TMyControlSubclass::AfterWndProc(Controls::TControl* Control, Messages::TMessage &Message)
{
 /* Do nothing */
}

void __fastcall TMyControlSubclass::doMouseDown(void)
{
  LBDown = true;
  OldP.x = Mouse->CursorPos.x;
  OldP.y = Mouse->CursorPos.y;
  OldQ = OldP;


  if (!JBForm->FUseTransparency)
    return;

  KillTimer(JBForm->Handle, 1);
  KillTimer(JBForm->Handle, 2);
  SetTimer(JBForm->Handle, 1, 20, NULL);
}

void __fastcall TMyControlSubclass::doMouseMove(void)
{
  if (!LBDown)
    return;

  Windows::TPoint Q;

  Q.x = Mouse->CursorPos.x;
  Q.y = Mouse->CursorPos.y;

  if ( Q.x != OldQ.x || Q.y != OldQ.y)
  {
    JBForm->Left += Q.x - OldQ.x;
    JBForm->Top  += Q.y - OldQ.y;
  }

  OldQ = Q;
}


void __fastcall TMyControlSubclass::doNcMouseMove(void)
{
  if (Timer1 || Timer2)
    return;

  if (!LBDown)
   return;

  LBDown = false;

  if (!JBForm->FUseTransparency)
    return;

  KillTimer(JBForm->Handle, 1);
  KillTimer(JBForm->Handle, 2);
  SetTimer(JBForm->Handle, 2, 20, NULL);
}


void __fastcall TMyControlSubclass::doMouseUp(void)
{

  float SnapMargin;
  LBDown = false;

  if (! JBForm->FUseTransparency)
    return;

  KillTimer(JBForm->Handle, 1);
  KillTimer(JBForm->Handle, 2);
  SetTimer(JBForm->Handle, 2, 20, NULL);

  if (!JBForm->FSnapToEdge)
    return;

  SnapMargin = JBForm->FMargin / 100.0f;

  if (Screen->Width - (JBForm->Left + JBForm->ClientWidth) <= (SnapMargin * (Screen->Width)) )
    MoveWindow(JBForm->Handle, Screen->Width - JBForm->ClientWidth, JBForm->Top, JBForm->Width, JBForm->Height, true);

  if (Screen->Height - (JBForm->Top + JBForm->ClientHeight) <= (SnapMargin * (Screen->Height)))
    MoveWindow(JBForm->Handle, JBForm->Left, Screen->Height - JBForm->ClientHeight, JBForm->Width, JBForm->Height, true);

  if (JBForm->Left <= (Screen->Width * SnapMargin))
    MoveWindow(JBForm->Handle, 0, JBForm->Top, JBForm->Width, JBForm->Height, True);

  if (JBForm->Top <= (Screen->Height * SnapMargin))
    MoveWindow(JBForm->Handle, JBForm->Left, 0, JBForm->Width, JBForm->Height, True);
}

void __fastcall TMyControlSubclass::doBlendUp(void)
{
  if (JBForm->Transparency == 255)
    return;

  JBForm->Transparency += 10;
  if (JBForm->Transparency > 255)
    JBForm->Transparency = 255;

  if (JBForm->Transparency == 255)
     KillTimer(JBForm->Handle, 2);
}

void __fastcall TMyControlSubclass::doBlendDown(void)
{
  JBForm->Transparency -= 10;
  if (JBForm->Transparency < 155)
     KillTimer(JBForm->Handle, 1);
}

__fastcall TMyControlSubclass::TMyControlSubclass(void)
{
  LBDown = false;
  Timer1 = false;
  Timer2 = false;
}

} 	/* namespace Unitjbform */
//---------------------------------------------------------------------------
#pragma package(smart_init)
